﻿#include "HelloWorldScene.h"

USING_NS_CC;
using namespace std; // Nếu ko dùng namespace này, khai báo string str sẽ lỗi

#define PTM_RATIO 32  
#define DEGTORAD (3.141592653589793 / 180.0)
#define PI 3.141592653589793

Scene* HelloWorld::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
	if (!Layer::init())
	{
		return false;
	}

	// Lớp Point đã được thay = Vec2 từ V3.1

	

	Size visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	setTouchEnabled(true);
	Size screenSize = Director::getInstance()->getWinSize();

	// Khởi tạo world của Box2D
	b2Vec2 gravity;
	gravity.Set(0.0f, -10.0f);
	bool doSleep = true;
	world = new b2World(gravity);
	world->SetAllowSleeping(doSleep);
	world->SetContinuousPhysics(true);

	this->debugDraw = new GLESDebugDraw(PTM_RATIO);
	this->world->SetDebugDraw(debugDraw);


	uint32  flags = 0;
	flags += b2Draw::e_shapeBit;                          
	flags += b2Draw::e_jointBit;
	flags += b2Draw :: e_aabbBit; 
	flags += b2Draw :: e_pairBit;
	flags += b2Draw::e_centerOfMassBit;

	//Tạo Ground bao quanh màn hình
	b2BodyDef groundBodyDef;


	groundBodyDef.position.Set(screenSize.width / 2 / PTM_RATIO,
		screenSize.height / 2 / PTM_RATIO);

	groundBody = world->CreateBody(&groundBodyDef);

	b2PolygonShape groundBox;

	// bottom
	groundBox.SetAsBox(screenSize.width / 2 / PTM_RATIO, 0, b2Vec2(0, -screenSize.height / 2 / PTM_RATIO), 0);
	groundBody->CreateFixture(&groundBox, 0);
	// top
	//groundBox.SetAsBox(screenSize.width / 2 / PTM_RATIO, 0, b2Vec2(0, screenSize.height / 2 / PTM_RATIO), 0);
	//groundBody->CreateFixture(&groundBox, 0);

	// left
	groundBox.SetAsBox(0, screenSize.height / 2 / PTM_RATIO, b2Vec2(-screenSize.width / 2 / PTM_RATIO, 0), 0);
	groundBody->CreateFixture(&groundBox, 0);

	// right
	groundBox.SetAsBox(0, screenSize.height / 2 / PTM_RATIO, b2Vec2(screenSize.width / 2 / PTM_RATIO, 0), 0);
	groundBody->CreateFixture(&groundBox, 0);

	//--------------------------------------
	// Bắt sự kiện Touch

	auto touchListener = EventListenerTouchOneByOne::create();
	touchListener->setSwallowTouches(true);

	touchListener->onTouchBegan = CC_CALLBACK_2(HelloWorld::onTouchBegan, this);
	touchListener->onTouchMoved = CC_CALLBACK_2(HelloWorld::onTouchMoved, this);
	touchListener->onTouchEnded = CC_CALLBACK_2(HelloWorld::onTouchEnded, this);
	_eventDispatcher->addEventListenerWithSceneGraphPriority(touchListener, this);

	
	this->debugDraw->SetFlags(flags);

	//***********************************************************************************
	//mouseJoint();
	//revoluteJoint();
	pulleyJoint();
	schedule(schedule_selector(HelloWorld::update));

	return true;
}

void HelloWorld::update(float dt) {
	int velocityIterations = 8;
	int positionIterations = 1;
	world->Step(dt, velocityIterations, positionIterations);

	for (b2Body *body = world->GetBodyList(); body != NULL; body = body->GetNext())
	if (body->GetUserData())
	{
		Sprite *sprite = (Sprite *)body->GetUserData();
		sprite->setPosition(Point(body->GetPosition().x * PTM_RATIO, body->GetPosition().y * PTM_RATIO));
		sprite->setRotation(-1 * CC_RADIANS_TO_DEGREES(body->GetAngle()));

	}
	world->ClearForces();
	world->DrawDebugData();

}

void HelloWorld::onTouchEnded(Touch* touch, Event* event) {
	if (m_mouseJoint) {
		world->DestroyJoint(m_mouseJoint);
		m_mouseJoint = NULL;
	}
}

bool HelloWorld::onTouchBegan(Touch* touch, Event* event) {
	auto location = touch->getLocation();
	b2Vec2 locationWorld = b2Vec2(location.x / PTM_RATIO, location.y / PTM_RATIO);

	if (m_bodyAFixtures->TestPoint(locationWorld)) {
		b2MouseJointDef md;
		md.bodyA = groundBody;
		md.bodyB = m_bodyA;
		md.target = locationWorld;
		md.collideConnected = true;
		md.maxForce = 1000.0f * m_bodyA->GetMass();

		m_mouseJoint = (b2MouseJoint *)world->CreateJoint(&md);
		m_bodyA->SetAwake(true);
	}
	
	return true;
}

void HelloWorld::onTouchMoved(Touch* touch, Event* event) {
	if (m_mouseJoint == NULL) return;

	auto location = touch->getLocation();
	b2Vec2 locationWorld = b2Vec2(location.x / PTM_RATIO, location.y / PTM_RATIO);

	m_mouseJoint->SetTarget(locationWorld);
}

void HelloWorld::draw(Renderer *renderer, const Mat4 &transform, uint32_t flags)
{
	Layer::draw(renderer, transform, flags);
	Director* director = Director::getInstance();
	GL::enableVertexAttribs(GL::VERTEX_ATTRIB_FLAG_POSITION );
	director->pushMatrix(MATRIX_STACK_TYPE::MATRIX_STACK_MODELVIEW);
	world->DrawDebugData();
	director->popMatrix(MATRIX_STACK_TYPE::MATRIX_STACK_MODELVIEW);
}

void HelloWorld::menuCloseCallback(Ref* pSender)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
	MessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
    return;
#endif

    Director::getInstance()->end();

#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
}


void HelloWorld::revoluteJoint() {
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1;

	b2PolygonShape boxShape;
	boxShape.SetAsBox(5, 1);
	b2CircleShape circleShape;
	circleShape.m_radius = 1;

	bodyDef.position.Set(500 / PTM_RATIO, 500 / PTM_RATIO);
	fixtureDef.shape = &boxShape;
	m_bodyA = world->CreateBody(&bodyDef);
	m_bodyAFixtures = m_bodyA->CreateFixture(&fixtureDef);

	bodyDef.position.Set(500 / PTM_RATIO, 500 / PTM_RATIO);
	bodyDef.type = b2_staticBody;
	fixtureDef.shape = &circleShape;
	m_bodyB = world->CreateBody(&bodyDef);
	m_bodyB->CreateFixture(&fixtureDef);

	b2RevoluteJointDef revoluteJointDef;

	revoluteJointDef.enableMotor = true;
	revoluteJointDef.maxMotorTorque = 200;
	revoluteJointDef.motorSpeed = 100;

	revoluteJointDef.bodyA = m_bodyA;
	revoluteJointDef.bodyB = m_bodyB;

	//revoluteJointDef.enableLimit = true;
	//revoluteJointDef.lowerAngle = -90* PI / 180;
	//revoluteJointDef.upperAngle = 90 * PI / 180;

	revoluteJointDef.collideConnected = true;
	revoluteJointDef.localAnchorA.Set(2, 0);
	revoluteJointDef.localAnchorB.Set(0, 0);
	m_revoluteJoint = (b2RevoluteJoint*)world->CreateJoint(&revoluteJointDef);

}

void HelloWorld::mouseJoint() {
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1;

	b2PolygonShape boxShape;
	boxShape.SetAsBox(100 / PTM_RATIO, 100 / PTM_RATIO);

	bodyDef.position.Set(400 / PTM_RATIO, 400 / PTM_RATIO);
	fixtureDef.shape = &boxShape;
	m_bodyA = world->CreateBody(&bodyDef);
	m_bodyAFixtures = m_bodyA->CreateFixture(&fixtureDef);
}

void HelloWorld::pulleyJoint() {
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1;

	b2PolygonShape boxShape;
	boxShape.SetAsBox(1, 1);
	b2CircleShape circleShape;
	circleShape.m_radius = 1;

	bodyDef.position.Set(300 / PTM_RATIO, 200 / PTM_RATIO);
	fixtureDef.shape = &boxShape;
	m_bodyA = world->CreateBody(&bodyDef);
	m_bodyAFixtures = m_bodyA->CreateFixture(&fixtureDef);

	bodyDef.position.Set(500 / PTM_RATIO, 200 / PTM_RATIO);
	bodyDef.type = b2_dynamicBody;
	fixtureDef.shape = &circleShape;
	m_bodyB = world->CreateBody(&bodyDef);
	m_bodyB->CreateFixture(&fixtureDef);

	b2Vec2 anchor1 = m_bodyA->GetWorldCenter();
	b2Vec2 anchor2 = m_bodyB->GetWorldCenter();
	b2Vec2 groundAnchor1 = b2Vec2(anchor1.x, anchor1.y + (300 / PTM_RATIO));
	b2Vec2 groundAnchor2 = b2Vec2(anchor2.x, anchor2.y + (300 / PTM_RATIO));
	float ratio = 1.0f;

	b2PulleyJointDef revoluteJointDef;

	revoluteJointDef.Initialize(m_bodyA, m_bodyB, groundAnchor1, groundAnchor2, anchor1, anchor2, ratio);
	revoluteJointDef.lengthA = 200 / PTM_RATIO;
	revoluteJointDef.lengthB = 200 / PTM_RATIO;

	m_pulleyJoint = (b2PulleyJoint*)world->CreateJoint(&revoluteJointDef);
}